import time
import sys
import re
import os

import ee
import geemap

from sepal_ui.scripts import utils, gee
from sepal_ui.scripts import messages as ms

#initialize earth engine
ee.Initialize()

def isAsset(asset_descripsion, folder):
    """Check if the asset already exist in the user asset folder
    
    Args: 
        asset_descripsion (str) : the descripsion of the asset
        folder (str): the folder of the glad assets
        
    Returns:
        exist (bool): true if already in folder
    """
    exist = False
    liste = ee.data.listAssets({'parent': folder})['assets']
    for asset in liste:
        if asset['name'] == folder + asset_descripsion:
            exist = True
            break
    return exist    

def run_aoi_selection(file_input, file_name, country_selection, asset_name, drawing_method, widget_alert, list_method, drawn_feat):
    """ create an gee asset according to the user inputs

    Args:
        file_input (str): the file to retreive from the user folder. It must be a .shp file
        file_name (str): name of the aoi that will be used troughout the process
        country_selection (str): a country name in english available in LSIB 2017
        asset_name (str): the assetId of a existing asset
        drawing_method (str): the name of the method selected to create the asset
        widget_alert (v.Alert): the widget used to display the process informations
        list_method ([str]): list of the method use to select an AOI
        drawn_feat (ee.FeatureCollection): the last drawn object on the map
        
    Returns:
        asset (str) : the AssetId of the AOI
    """
    #go to the glad folder in gee assets 
    folder = ee.data.getAssetRoots()[0]['id'] + '/'
    
    #check the drawing method
    if drawing_method == None: #not selected
        utils.displayIO(widget_alert, ms.NO_SELECTION, 'warning')    
        asset = None
        
    elif drawing_method == list_method[0]: #use a country boundary
        if country_selection == None:
            utils.displayIO(widget_alert, ms.NO_COUNTRY, 'warning') 
            asset = None
            return asset
        country_code = utils.create_FIPS_dic()[country_selection] 
        asset_descripsion = ms.FILE_PATTERN.format(country_code) 
        asset = folder + asset_descripsion
            
        #check asset existence
        if isAsset(asset_descripsion, folder):
            utils.displayIO(widget_alert, ms.ASSET_ALREADY_EXIST.format(asset), 'success')
            return asset
        
        country = ee.FeatureCollection('USDOS/LSIB_SIMPLE/2017').filter(ee.Filter.eq('country_co', country_code))
                
        #create and launch the task
        task_config = {
            'collection': country, 
            'description':asset_descripsion,
            'assetId': asset
        }
        task = ee.batch.Export.table.toAsset(**task_config)
        task.start()
        gee.wait_for_completion(asset_descripsion, widget_alert)
          
        utils.displayIO(widget_alert, ms.ASSET_CREATED.format(asset), 'success')
                
    elif drawing_method == list_method[1]: #draw a shape
             
        aoi = drawn_feat 
        asset_name = ms.FILE_PATTERN.format(re.sub('[^a-zA-Z\d\-\_]', '_', file_name))
        
        #check if something is drawn 
        if drawn_feat == None:
            asset = None
            utils.displayIO(widget_alert, ms.NO_SHAPE, 'error')
            return asset
              
        #check asset existence
        if isAsset(asset_name, folder):
            asset = None
            utils.displayIO(widget_alert, ms.NAME_USED, 'error')
            return asset 
        
        asset = folder + asset_name
            
        #create and launch the task
        task_config = {
            'collection': aoi, 
            'description':asset_name,
            'assetId': asset
        }
        task = ee.batch.Export.table.toAsset(**task_config)
        task.start()
        gee.wait_for_completion(asset_name, widget_alert)
           
        utils.displayIO(widget_alert, ms.ASSET_CREATED.format(asset), 'success')           
            
    elif drawing_method == list_method[3]: #use GEE asset
        
        #verify that there is an asset
        if asset_name == '' or asset_name == None:
            asset = None
            utils.displayIO(widget_alert, ms.NO_ASSET, 'error') 
        else:
            asset = asset_name
            utils.displayIO(widget_alert, ms.CHECK_IF_ASSET, 'info')
            
    elif drawing_method == list_method[2]: #upload file
        home_path = os.path.expanduser('~')
        file_input = home_path + '/' + file_input
        print(file_input)
        try:
            ee_object = geemap.shp_to_ee(file_input)
        except:
            utils.displayIO(widget_alert, ms.ERROR_OCCURED, 'error')
            asset = None
            return asset
        
        name = os.path.split(file_input)[1]
        
        asset_name = ms.FILE_PATTERN.format(re.sub('[^a-zA-Z\d\-\_]','_',name))
        
        #check asset's name
        if isAsset(asset_name, folder):
            asset = None
            utils.displayIO(widget_alert, ms.NAME_USED, 'error')
        else:
            asset = folder + asset_name
            
            #launch the task
            task_config = {
                'collection': ee_object, 
                'description':asset_name,
                'assetId': asset
            }
            task = ee.batch.Export.table.toAsset(**task_config)
            task.start()
            gee.wait_for_completion(asset_name, widget_alert)
                   
            utils.displayIO(widget_alert, ms.ASSET_CREATED.format(asset), 'success')
            
    return asset


from markdown import markdown
from datetime import datetime
import traitlets

import ipyvuetify as v

from sepal_ui import widgetBinding as wb
from sepal_ui.scripts import mapping, utils


sepal_main = '#2e7d32'
sepal_darker = '#005005'

#create a app bar 
def AppBar(title='SEPAL module'):
    """
    create an appBar widget with the provided title using the sepal color framework
    
    Returns: 
        app (v.AppBar) : The created appbar
        toolbarButton (v.Btn) : The button to display the side drawer
    """

    toolBarButton = v.Btn(
        icon = True, 
        children=[
            v.Icon(class_="white--text", children=['mdi-dots-vertical'])
        ]
    )
    
    appBar = v.AppBar(
        color=sepal_main,
        class_="white--text",
        dense=True,
        app = True,
        children = [
            toolBarButton, 
            v.ToolbarTitle(children=[title]),
        ]
    )
    
    return (appBar, toolBarButton)
    
#create a drawer_item
def DrawerItem(title, icon=None, card='', href=''):
    """ 
    create a drawer item using the user input
    
    Args:
        title (str): the title to display in the drawer,
        icon (str, optional): the icon id following the mdi code. folder icon if None
        card (str), optional): the tile metadata linked to the drawer
        href(str, optional): the link targeted by the button
    
    Returns:
        item (v.ListItem): the item to display
    """
    
    if not icon:
        icon = 'mdi-folder-outline'
        
    item = v.ListItem(
        link=True,
        children=[
            v.ListItemAction(
                children=[
                    v.Icon(
                        class_="white--text", 
                        children=[icon])
                ]
            ),
            v.ListItemContent(
                children=[
                    v.ListItemTitle(
                        class_="white--text", 
                        children=[title]
                    )
                ]
            )
        ]
    )
    
    if not href == '':
        item.href=href
        item.target="_blank"

    if not card == '':
        item._metadata = {'card_id': card }
    
    return item

#create a drawer 
def NavDrawer(items, code = False, wiki = False, issue = None):
    """ 
    create a navdrawer using the different items of the user and the sepal color framework. The drawer always include links to the github page of the project for wiki, bugs and repository.
    
    Args:
        items ([v.ListItem]) : the list of the list item the user wants to add to the nav drawer
        code (str, optionnal) : the absolute link to the github code. not display if None
        wiki (str, optionnal) : the absolute link to the github wiki. not display if None
        issue (str, optionnal) : the absolute link to the github issues. not display if None
    
    Returns:
        navDrawer (v.NavigationDrawer) : the nav drawer of the web page
    """
    
    code_link = []
    if code:
        item_code = DrawerItem('Source code', icon='mdi-file-code', href=code)
        code_link.append(item_code)
    if wiki:
        item_wiki = DrawerItem('Wiki', icon='mdi-book-open-page-variant', href=wiki)
        code_link.append(item_wiki)
    if issue:
        item_bug = DrawerItem('Bug report', icon='mdi-bug', href=issue)
        code_link.append(item_bug)
        
    
        
    navDrawer = v.NavigationDrawer(
        v_model=True, 
        app= True,
        color=sepal_darker,
        children=[
            v.List(dense=True, children=items),
            v.Divider(),
            v.List(dense=True, children=code_link),
        ]
    )
    
    return navDrawer


#create a footer 
def Footer (text='SEPAL \u00A9 2020'): 
    """ 
    create a footer with cuzomizable text. Not yet capable of displaying logos
    
    Returns: 
        footer (v.Footer) : an app footer
    """
    
    footer = v.Footer(
        color=sepal_main,
        class_="white--text",
        app = True,
        children = [text]
    )
    
    return footer


#create an app
def App (tiles=[''], appBar=None, footer=None, navDrawer=None):
    """
    Create an app display with the tiles created by the user. Display false footer and appBar if not filled. navdrawer is fully optionnal
    
    Args:
        tiles ([v.Layout]) : the list of tiles the user want to display in step order. 
        appBar (v.appBar, optionnal) : the custom appBar of the module
        footer (v.Footer, optionnal) : the custom footer of the module
        navDrawer (v.NavigationDrawer, optional) : the navigation drawer to allow the display of specific tiles
        
    Returns:
        app (v.App) : the complete app to display
        toolBarButton (v.Btn) : the created toolbarButton, None if the appBar was already existing
    """
    
    app = v.App(v_model=None)
    app_children = []
    
    #add the navDrawer if existing
    if navDrawer:
        app_children.append(navDrawer)
    
    #create a false appBar if necessary
    toolBarButton = None
    if not appBar:
        appBar, toolBarButton = AppBar()
    app_children.append(appBar)

    #add the content of the app
    content = v.Content(children=[v.Container(fluid=True,children = tiles)])
    app_children.append(content)
    
    #create a false footer if necessary
    if not footer:
        footer = Footer()
    app_children.append(footer)

    app.children = app_children
    
    return (app, toolBarButton)

#create a tile 
def Tile(id_, title, inputs=[''], btn=None, output=None):
    """ 
    create a customizable tile for the sepal UI framework
    
    Args: 
        id_ (str) : the Id you want to gave to the tile. This Id will be used by the draweritems to show and hide the tile.
        title (str) : the title that will be display on the top of the tile
        btn (v.Btn, optionnal) : if the tile launch a py process, attached a btn to it.
        output( v.Alert, optional) : if you want to display text results of your process add an alert widget to it
        
    Returns: 
        tile (v.Layout) : a fully functionnal tile to be display in an app
    """
    
    if btn:
        inputs.append(btn)
    
    if output:
        inputs.append(output)
        
    
    inputs_widget = v.Layout(
        _metadata={'mount-id': '{}-data-input'.format(id_)},
        row=True,
        class_="pa-5",
        align_center=True, 
        children=[v.Flex(xs12=True, children=[widget]) for widget in inputs]
    )
    
    tile = v.Layout(
        _metadata={'mount_id': id_},
        row=True,
        xs12=True,
        align_center=True, 
        class_="ma-5 d-inline",
        children=[
            v.Card( 
                class_="pa-5",
                raised=True,
                xs12=True,
                children=[
                    v.Html(xs12=True, tag='h2', children=[title]),
                    v.Flex(xs12=True, children=[inputs_widget]),   
                ]
            )
        ]
    )
    
    return tile

#create the about tile 
def TileAbout(pathname):
    """
    create a about tile using a md file. This tile will have the "about_widget" id and "About" title.
    
    Args:
        pathname (str) : the pathname to the .md file of the about section
        
    Returns:
        tile (v.Layout) : a about tile
    """
    
    #read the content and transform it into a html
    f = open(pathname, 'r')
    if f.mode == 'r':
        about = f.read()
    else :
        about = '**No About File**'
        
    about = markdown(about, extensions=['fenced_code', 'sane_lists'])
    
    #need to be nested in a div to be displayed
    about = '<div>\n' + about + '\n</div>'
    
    #create a Html widget
    class MyHTML(v.VuetifyTemplate):
        template = traitlets.Unicode(about).tag(sync=True)
    
    
    content = MyHTML()
    
    #content = w.HTML(value=about)
    
    return Tile('about_widget', 'About', inputs=[content])
    

#create downloadable links button 
def DownloadBtn(text, url="#"):
    btn = v.Btn(
        class_='ma-2',
        xs5=True,
        color='success',
        href=url,
        children=[
            v.Icon(left=True, children=['mdi-download']),
            text
        ]
    )
    
    return btn

#create a process button 
def ProcessBtn(text="Launch process"):
    """
    create a process button filled with the provided text
    
    Returns: 
        btn (v.Btn) : the btn
    """
    btn = v.Btn(
        color='primary', 
        children=[
            v.Icon(left=True, children=['mdi-map-marker-check']),
            text
        ]
    )
    
    return btn

#hide all the cards but one for the start of the notebook
def hideCards(tileId, tiles):
    """
    hide all the cards but the on selected
    
    Args: 
        tileId (str) : the card to be display when opening the app 
        tiles ([v.cards]) : a list of the cards in the app
    """
    for tile in tiles:
            if tileId == tile._metadata['mount_id']:
                tile.class_="ma-5 d-inline"
            else:
                tile.class_="ma-5 d-none"
    

#create the aoi selectors IO
class Aoi_IO:
    def __init__(self):
        #set up your inputs
        self.file_input = None
        self.file_name = 'Manual_{0}'.format(datetime.now().strftime("%Y-%m-%d_%H-%M-%S"))
        self.country_selection = None
        self.selection_method = None
        self.drawn_feat = None
    
        #set up your output
        self.assetId = None

#create an aoi selector tile with all its bindings
def TileAOI(io):
    """render and bind all the variable to create an autonomous aoi selector. It will create a asset in you gee account with the name 'aoi_[aoi_name]'. The assetId will be added to io.assetId.
    
    Args: 
        io (Aoi_IO) : an Aoi_IO object that content all the IO of the aoi selector tile in your app
        
   Returns:
       tile (v.Layout) : an autonomous tile for AOI selection binded with io
   """
    
    #constants
    selection_method = [
        'Country boundaries', 
        'Draw a shape', 
        'Upload file', 
        'Use GEE asset'
    ]
    AOI_MESSAGE='Click on "Selet these inputs" to validate your AOI'
    
    #create the output
    aoi_output = OutputWidget(AOI_MESSAGE)
    
    #create the inputs widgets 
    aoi_file_input = v.Select(
        items=utils.get_shp_files(), 
        label='Select a file', 
        v_model=None,
        class_='d-none'
    )
    wb.bind(aoi_file_input, io, 'file_input', aoi_output)
    
    aoi_file_name = v.TextField(
        label='Select a filename', 
        v_model=io.file_name,
        class_='d-none'
    )
    wb.bind(aoi_file_name, io, 'file_name', aoi_output)
    
    aoi_country_selection = v.Select(
        items=[*utils.create_FIPS_dic()], 
        label='Country/Province', 
        v_model=None,
        class_='d-none'
    )
    wb.bind(aoi_country_selection, io, 'country_selection', aoi_output)
    
    aoi_asset_name = v.TextField(
        label='Select a GEE asset', 
        v_model=None,
        class_='d-none'
    )
    wb.bind(aoi_asset_name, io, 'assetId', aoi_output)
    
    widget_list = [aoi_file_input, aoi_file_name, aoi_country_selection, aoi_asset_name]
    
    #create the map 
    dc, m = mapping.init_map()
    wb.handle_draw(dc, io, 'drawn_feat', aoi_output)
    
    #bind the input to the selected method 
    aoi_select_method = v.Select(items=selection_method, label='AOI selection method', v_model=None)
    wb.bindAoiMethod(aoi_select_method, widget_list, io, m, dc, selection_method)
    

    #create the validation button 
    aoi_select_btn = ProcessBtn('Select these inputs')
    wb.bindAoiProcess(aoi_select_btn, io, m, dc, aoi_output, selection_method)
    
    #assemble everything on a tile 
    inputs = v.Layout(
        _metadata={'mount-id': 'data-input'},
        class_="pa-5",
        row=True,
        align_center=True, 
        children=[
            v.Flex(xs12=True, children=[aoi_select_method]),
            v.Flex(xs12=True, children=[aoi_country_selection]),
            v.Flex(xs12=True, children=[aoi_file_input]),
            v.Flex(xs12=True, children=[aoi_file_name]),
            v.Flex(xs12=True, children=[aoi_asset_name]),
            v.Flex(xs12=True, children=[aoi_select_btn]),
            v.Flex(xs12=True, children=[aoi_output]),
        ]
    )

    AOI_content_main =  v.Layout(
        _metadata={'mount_id': 'aoi_widget'},
        xs12=True,
        row=True,
        class_="ma-5 d-inline",
        children=[
            v.Card(
                class_="pa-5",
                raised=True,
                xs12=True,
                children=[
                    v.Html(xs12=True, tag='h2', children=['AOI selection']),
                    v.Layout(
                        row=True,
                        xs12=True,
                        children=[
                            v.Flex(xs12=True, md6=True, children=[inputs]),
                            v.Flex(class_="pa-5", xs12=True, md6=True, children=[m])
                        ]
                    )    
                ]
            )
        ]
    )
    
    return AOI_content_main

#create an alert 
def OutputWidget(text='Click it'):
    """
    create an output alert that can be used to display the process outputs
    """
    
    alert = v.Alert(children=[text], type='info', text=True, class_="mt-5")
    
    return alert
    
    
    



from functools import partial

import ipyvuetify as v
import geemap
import ee

from sepal_ui.scripts import utils, run_aoi_selection, mapping
 

ee.Initialize()

#make the toolbar button clickable 
def displayDrawer(drawer, toggleButton):
    """
    bin the drawer to it's toggleButton
    
    Args:
        drawer (v.navigationDrawer) : the drawer tobe displayed
        toggleButton(v.Btn) : the button that activate the drawer
    """
    def on_click(widget, event, data, drawer):
        drawer.v_model = not drawer.v_model
        
    toggleButton.on_event('click', partial(on_click, drawer=drawer))

#display the appropriate tile
def display_tile(item, tiles):
    """
    display the apropriate tiles when the item is clicked
    
    Args:
        item (v.ListItem) : the item in the drawer that select the active tile
        tiles ([v.Layout]) : the list of all the available tiles in the app
    """
    def on_click(widget, event, data, tiles):
        for tile in tiles:
            if widget._metadata['card_id'] == tile._metadata['mount_id']:
                tile.class_="ma-5 d-inline"
            else:
                tile.class_="ma-5 d-none"
    
    item.on_event('click', partial(on_click, tiles=tiles))

def bind(widget, obj, variable, output=None, output_message='The selected variable is: '):
    """ 
    bind the variable to the widget
    
    Args:
        widget (v.XX) : an ipyvuetify input element
        obj : the process_io object
        variable (str) : the name of the member in process_io object
        output (v.Alert, opotional) : the alert element to display the variable new value
        output_message (str, optionnal) : the output message before the variable display
    """
    
    def on_change(widget, event, data, obj, variable, output, output_message):
        
        setattr(obj, variable, widget.v_model)
        if output:
            message = output_message + str(widget.v_model)
            utils.displayIO(output, message)
        
    widget.on_event('change', partial(
        on_change,
        obj=obj,
        variable=variable, 
        output=output, 
        output_message=output_message
    ))
    
def toggle_inputs(input_list, widget_list):
    """
    display only the widgets that are part of the input_list. the widget_list is the list of all the widgets of the tile.
    
    Args:
        input_list ([v.widget]) : the list of input to be display
        widget_list ([v.widget]) : the list of the tile widget
    """
    
    for input_item in widget_list:
        if input_item in input_list:
            input_item.class_ = 'd-inline'
        else:
            input_item.class_ = 'd-none'

    return

def bindAoiMethod(method_widget, list_input, obj, m, dc, selection_method):
    """
    change the display of the AOI selector according to the method selected. will only display the useful one
    
    Args: 
        method_widget (v.select) : the method selector widget 
        list_input ([v.widget]) : the list of all the aoi inputs
        obj (Aoi_IO) : the IO object of the tile
        m (geemap.Map) the map displayed in the tile
        dc (DrawControl) : the drawing control
        selection_method ([str]) : the available selection methods
    """
    
    def on_change(widget, event, data, list_input, obj, m, dc, selection_method):
        
        #clearly identify the differents widgets 
        aoi_file_input = list_input[0]
        aoi_file_name = list_input[1]
        aoi_country_selection = list_input[2]
        aoi_asset_name = list_input[3]
        
        setattr(obj, 'selection_method', widget.v_model)
        
        #remove the map 
        try:
            m.remove_control(dc)
        except:
            pass
        dc.clear()
        #toogle the appropriate inputs
        if widget.v_model == selection_method[0]: #country selection
            toggle_inputs([aoi_country_selection], list_input)
        elif widget.v_model == selection_method[1]: #drawing
            toggle_inputs([aoi_file_name], list_input)
            m.add_control(dc)
        elif widget.v_model == selection_method[2]: #shp file
            toggle_inputs([aoi_file_input], list_input)
        elif widget.v_model == selection_method[3]: #gee asset
            toggle_inputs([aoi_asset_name], list_input)
    
    
    
    method_widget.on_event('change', partial(
        on_change,
        list_input=list_input,
        obj=obj,
        m=m,
        dc=dc, 
        selection_method=selection_method
    ))
    
    return 

# Handle draw events
def handle_draw(dc, obj, variable, output=None):
    """ 
    handle the drawing of a geometry on a map. The geometry is transform into a ee.featurecollection and send to the variable attribute of obj.
    
    Args: 
        dc (DrawControl) : the draw control on which the drawing will be done 
        obj (obj IO) : any object created for IO of your tile 
        variable (str) : the name of the atrribute of the obj object where to store the ee.FeatureCollection 
        output (v.Alert, optionnal) : the output to display results
        
    """
    def on_draw(self, action, geo_json, obj, variable, output):
        geom = geemap.geojson_to_ee(geo_json, False)
        feature = ee.Feature(geom)
        setattr(obj, variable, ee.FeatureCollection(feature)) 
        
        if output:
            utils.displayIO(output, 'A shape have been drawn')
        
        return 
        
        
    dc.on_draw(partial(
        on_draw,
        obj=obj,
        variable=variable,
        output=output
    ))
    
    return

def bindAoiProcess(btn, io, m, dc, output, list_method):
    """
    Create an asset in your gee acount and serve it to the map.
    
    Args:
        btn (v.Btn) : the btn that launch the process
        io (Aoi_IO) : the IO of the aoi selection tile
        m (geemap.Map) : the tile map
        dc (drawcontrol) : the drawcontrol
        output (v.Alert) : the alert of the selector tile
        list_method([str]) : the list of the available selections methods
    """
    
    def on_click(widget, event, data, io, m, dc, output, list_method):
        
        utils.toggleLoading(widget)
        
        #create the aoi asset
        assetId = run_aoi_selection.run_aoi_selection(
            file_input        = io.file_input, 
            file_name         = io.file_name, 
            country_selection = io.country_selection, 
            asset_name        = io.assetId, 
            drawn_feat        = io.drawn_feat,
            drawing_method    = io.selection_method,
            widget_alert      = output, 
            list_method       = list_method, 
        )
        
        #remove the dc
        dc.clear()
        try:
            m.remove_control(dc)
        except:
            pass
        
        #display it on the map
        if assetId:
            mapping.update_map(m, dc, assetId)
            
        utils.toggleLoading(widget)
        
        #add the value to the IO object 
        setattr(io, 'assetId', assetId)
        
        return 
    
    btn.on_event('click', partial(
        on_click,
        io          = io, 
        m           = m, 
        dc          = dc, 
        output      = output,
        list_method = list_method
    ))
    
    return


import time

import ee

from sepal_ui.scripts import utils
from sepal_ui.scripts import messages as ms

ee.Initialize()

def wait_for_completion(task_descripsion, widget_alert):
    """Wait until the selected process is finished. Display some output information

    Args:
        task_descripsion (str) : name of the running task
        widget_alert (v.Alert) : alert to display the output messages
    """
    state = 'UNSUBMITTED'
    while state != 'COMPLETED':
        utils.displayIO(widget_alert, ms.STATUS.format(state))
        time.sleep(5)
                    
        #search for the task in task_list
        current_task = isTask(task_descripsion)
        state = current_task.state
        
def isTask(task_descripsion):
    """Search for the described task in the user Task list return None if nothing is find
    
    Args: 
        task_descripsion (str): the task descripsion
    
    Returns
        task (ee.Task) : return the found task else None
    """
    
    tasks_list = ee.batch.Task.list()
    current_task = None
    for task in tasks_list:
        if task.config['description'] == task_descripsion:
            current_task = task
            break
            
    return current_task





STATUS = "Status: {}"


###########################################
#####   AOI selector              #########
###########################################
FILE_PATTERN = 'aoi_{0}'
NO_SELECTION = 'No selection method have bee picked up'
NO_COUNTRY = 'No Country have been selected'
ASSET_ALREADY_EXIST = "The asset was already existing you can continue to use it. It's also available at :{0}"
ASSET_CREATED = "The asset has been created under the name : {0}"
NAME_USED = "The name was already in used, change it or delete the previous asset in your GEE acount"
NO_ASSET = "No Asset have been provided"
CHECK_IF_ASSET = "Check carefully that your string is an assetId"
NOT_AVAILABLE = "This function is not yet available"
NO_SHAPE = "No shape have been drawn on the map"
ERROR_OCCURED = "An error occured with provided .shp file"
AOI_MESSAGE='click on "selet these inputs" to validate your AOI'


import geemap
import ee 
import shapely.geometry as sg
from haversine import haversine

#initialize earth engine
ee.Initialize()

def init_map():
    '''Initialize a map centered on the point [0,0] with zoom at 1
    
    Returns:
        dc (DrawControl): a draw control for polygon drawing 
        m (Map): a map
    '''
    center = [0, 0]
    zoom = 2

    dc = geemap.DrawControl(
        marker={},
        circlemarker={},
        polyline={},
        rectangle={'shapeOptions': {'color': '#0000FF'}},
        circle={'shapeOptions': {'color': '#0000FF'}},
        polygon={'shapeOptions': {'color': '#0000FF'}},
     )

    m = geemap.Map(center=center, zoom=zoom)
    m.clear_layers()
    m.clear_controls()
    m.add_basemap('Esri Satellite')
    m.add_basemap('CartoDB.Positron')
    m.add_control(geemap.ZoomControl(position='topright'))
    m.add_control(geemap.LayersControl(position='topright'))
    m.add_control(geemap.AttributionControl(position='bottomleft'))
    m.add_control(geemap.ScaleControl(position='bottomleft', imperial=False))
    
    return (dc, m)

def update_map(m, dc, assetId):
    """Update the map with the asset overlay and by removing the selected drawing controls
    
    Args:
        m (Map): a map
        dc (DrawControl): the drawcontrol to be removed
        assetId (str): the asset ID in gee assets
    """  
    m.centerObject(ee.FeatureCollection(assetId), zoom=update_zoom(assetId))
    m.addLayer(ee.FeatureCollection(assetId), {'color': 'green'}, name='aoi')
    dc.clear()
    try:
        m.remove_control(dc)
    except:
        pass
    
def update_zoom(assetId):
    """search for the dimension of the AOI and adapt the map zoom acordingly
    
    Args: 
        assetId (str): the assetID
    
    Returns: 
        zoom (int): the zoom value riquired
    """
    
    #retreive the asset and transform it to shapely
    geom = ee.FeatureCollection(assetId).geometry()
    geomJson = geemap.ee_to_geojson(geom)
    geomShp = sg.shape(geomJson)
    
    #get the bounding box
    bb = {}
    bb['minx'], bb['miny'], bb['maxx'], bb['maxy'] = geomShp.bounds
    
    #create the 4 cardinal points
    tl = [bb['minx'], bb['maxy']]
    bl = [bb['minx'], bb['miny']]
    tr = [bb['maxx'], bb['maxy']]
    br = [bb['maxx'], bb['miny']]

    maxsize = max(haversine(tl, br), haversine(bl, tr))
    
    lg = 40075 #number of displayed km at zoom 1
    zoom = 1
    while lg > maxsize:
        zoom += 1
        lg /= 2
        
    return zoom-1


from distutils.core import setup
    
setup(
    name = 'sepal_ui',      
    packages = ['sepal_ui', 'sepal_ui.scripts'],   
    package_data={'sepal_ui': ['scripts/*.csv']},
    version = '0.2-beta',   
    license='MIT',        
    description = 'wrapper for ipyvuetify widgets to unify the display of voila dashboards in the sepal plateform',  
    author = 'Pierrick Rambaud',                   
    author_email = 'pierrick.rambaud49@gmail.com',  
    url = 'https://github.com/12rambau/sepal_ui',
    download_url = 'https://github.com/12rambau/sepal_ui/archive/v_0.2-beta.tar.gz',
    keywords = ['UI', 'Python', 'widget', 'sepal'], 
    install_requires=[
        'haversine',
        'ipyvuetify',
        'geemap',
        'ipyvuetify',
        'earthengine-api',
        'bqplot',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',  
        'Programming Language :: Python :: 3.6',
    ],
)


import time
from datetime import datetime
import os
import glob
from pathlib import Path
import csv

import ipyvuetify as v


def displayIO(widget_alert, message, alert_type='info'):
    """ Display the message in a vuetify alert DOM object with specific coloring
    Args: 
        widget_alert (v.Alert) : the vuetify alert to modify
        alert_type (str) : the alert color
        message (v.Children) : a DOM element or a string to fill the message 
    """
    
    list_color = ['info', 'success', 'warning', 'error']
    if not alert_type in list_color:
        alert_type = 'info'
        
    widget_alert.type = alert_type
     
    current_time = datetime.now().strftime("%Y/%m/%d, %H:%M:%S")
    widget_alert.children = [
        v.Html(tag='p', children=['[{0}]'.format(current_time)]),
        v.Html(tag='p', children=[message])
   ]
    
def toggleLoading(btn):
    """Toggle the loading state for a given btn in the ipyvutify lib
    
    Args:
        btn (v.Btn) : the btn to toggle
    """
    
    btn.loading = not btn.loading
    btn.disabled = btn.loading
    
def create_FIPS_dic():
    """create the list of the country code in the FIPS norm using the CSV file provided in utils
        
    Returns:
        fips_dic (dic): the country FIPS_codes labelled with english country names
    """
     
    pathname = os.path.join(os.path.dirname(__file__), 'FIPS_code_to_country.csv')
    fips_dic = {}
    with open(pathname, newline='') as f:
        reader = csv.reader(f, delimiter=';')
        next(reader)
        for row in reader:
            fips_dic[row[1]] = row[3]
            
        fips_sorted = {}
        for key in sorted(fips_dic):
            fips_sorted[key] = fips_dic[key]
        
    return fips_sorted

def get_shp_files():
    """return all the .shp files available in the user directories. Will verify if the .dbf and .shx exists and are located at the same place
    
    Returns: 
        shp_list (str[]): the path to every .shp complete and available, empty list if none
    """
    
    root_dir = os.path.expanduser('~')
    raw_list = glob.glob(root_dir + "/**/*.shp", recursive=True)
    
    #check if the file is complete
    shp_list = []
    for pathname in raw_list:
        path = Path(pathname)
        if os.path.isfile(path.with_suffix('.dbf')) and os.path.isfile(path.with_suffix('.shx')):
            shp_list.append(pathname)

    return shp_list